@@ -34,13 +34,17 @@
 
 #define DEBUG_LOCKS 0
 
+/* 
+ * See comments on async_poll_func() on the order of the various
+ * gets and sets of waiting_on_* flags.
+ */
 #if DEBUG_LOCKS
 #define DO_LOCK(mtx) G_STMT_START { \
 		g_printerr ("%s LOCK %s\n", __func__, G_STRINGIFY(mtx));  \
 		g_atomic_int_inc (&waiting_on_lock); \
 		if (g_atomic_int_get (&waiting_on_poll)) gkr_wakeup_now (); \
 		g_mutex_lock (mtx);  \
-		g_atomic_int_dec_and_test (&waiting_on_lock); \
+		g_atomic_int_exchange_and_add (&waiting_on_lock, -1); \
         } G_STMT_END
 #define DO_UNLOCK(mtx) G_STMT_START { \
 		g_printerr ("%s UNLOCK %s\n", __func__, G_STRINGIFY(mtx));  \
@@ -51,7 +55,7 @@
 		g_atomic_int_inc (&waiting_on_lock); \
 		if (g_atomic_int_get (&waiting_on_poll)) gkr_wakeup_now (); \
 		g_mutex_lock (mtx); \
-		g_atomic_int_dec_and_test (&waiting_on_lock); \
+		g_atomic_int_exchange_and_add (&waiting_on_lock, -1); \
 	} G_STMT_END
 #define DO_UNLOCK(mtx) \
 	g_mutex_unlock (mtx)
@@ -93,8 +97,18 @@ async_poll_func (GPollFD *ufds, guint nfsd, gint timeout)
 	gint ret;
 	
 	g_assert (orig_poll_func);
+
+	/* 
+	 * These two atomic variables are interlocked in the 
+	 * opposite order from those in DO_LOCK which prevents
+	 * race conditions in the if statements.
+	 */
 	g_atomic_int_set (&waiting_on_poll, 1);
+	if (g_atomic_int_get (&waiting_on_lock))
+		timeout = 0;
+
 	ret = (orig_poll_func) (ufds, nfsd, timeout);
+
 	g_atomic_int_set (&waiting_on_poll, 0);
 	
  	if (done_queue && !g_queue_is_empty (done_queue))
@@ -314,7 +328,6 @@ cleanup_done_thread (gpointer message, gpointer data)
 		running_workers = NULL;
 		
 		g_assert (main_loop);
-		gkr_wakeup_register (g_main_loop_get_context (main_loop));
 		return FALSE;
 	}
 	
@@ -346,7 +359,6 @@ gkr_async_worker_start (GThreadFunc func, GkrAsyncWorkerCallback callback,
 	
 	if (!done_queue) {
 		g_assert (main_loop);
-		gkr_wakeup_register (g_main_loop_get_context (main_loop));
 		
 		done_queue = g_queue_new ();
 		g_assert (!running_workers);
@@ -361,6 +373,10 @@ gkr_async_worker_start (GThreadFunc func, GkrAsyncWorkerCallback callback,
 	worker->cancelled = 0;
 	worker->stopped = 0;
 	
+	/* 
+	 * Don't change this to a thread pool too lightly. Assumptions are made 
+	 * that worker threads are not shared throughout the code.
+	 */
 	worker->thread = g_thread_create (async_worker_thread, worker, TRUE, &err);
 	if (!worker->thread) {
 		g_warning ("couldn't create new worker thread: %s", err->message);
@@ -600,7 +616,7 @@ gkr_async_usleep (gulong microseconds)
 	/* Let another worker or the main loop run */
 	DO_UNLOCK (async_mutex);
 	
-		g_usleep (G_USEC_PER_SEC);
+		g_usleep (microseconds);
 		
 	DO_LOCK (async_mutex);
 }
