@@ -32,7 +32,7 @@ typedef struct GP11Mechanism {
 
 typedef struct GP11Attribute {
 	gulong type;
-	gpointer value;
+	guchar *value;
 	gulong length;
 } GP11Attribute;
 
@@ -52,6 +52,9 @@ void                gp11_attribute_init                     (GP11Attribute *attr
                                                              gconstpointer value,
                                                              gsize length);
 
+void                gp11_attribute_init_invalid             (GP11Attribute *attr,
+                                                             guint attr_type);
+
 void                gp11_attribute_init_boolean             (GP11Attribute *attr,
                                                              guint attr_type,
                                                              gboolean value);
@@ -75,6 +78,8 @@ GP11Attribute*      gp11_attribute_new                      (guint attr_type,
                                                              gpointer value,
                                                              gsize length);
 
+GP11Attribute*      gp11_attribute_new_invalid              (guint attr_type);
+
 GP11Attribute*      gp11_attribute_new_boolean              (guint attr_type,
                                                              gboolean value);
 
@@ -87,13 +92,16 @@ GP11Attribute*      gp11_attribute_new_ulong                (guint attr_type,
 GP11Attribute*      gp11_attribute_new_string               (guint attr_type,
                                                              const gchar *value);
 
+gboolean            gp11_attribute_is_invalid               (GP11Attribute *attr);
+
 gboolean            gp11_attribute_get_boolean              (GP11Attribute *attr);
 
 gulong              gp11_attribute_get_ulong                (GP11Attribute *attr);
 
 gchar*              gp11_attribute_get_string               (GP11Attribute *attr);
 
-GDate*              gp11_attribute_get_date                 (GP11Attribute *attr);
+void                gp11_attribute_get_date                 (GP11Attribute *attr, 
+                                                             GDate* value);
 
 GP11Attribute*      gp11_attribute_dup                      (GP11Attribute *attr);
 
@@ -101,7 +109,12 @@ void                gp11_attribute_clear                    (GP11Attribute *attr
 
 void                gp11_attribute_free                     (GP11Attribute *attr);
 
+
 typedef struct _GP11Attributes GP11Attributes;
+
+#define             GP11_TYPE_ATTRIBUTES                    (gp11_attributes_get_boxed_type ())
+
+GType               gp11_attributes_get_boxed_type          (void) G_GNUC_CONST;
  
 GP11Attributes*     gp11_attributes_new                     (void);
 
@@ -124,6 +137,9 @@ void                gp11_attributes_add_data                (GP11Attributes *att
                                                              gconstpointer value,
                                                              gsize length);
 
+void                gp11_attributes_add_invalid             (GP11Attributes *attrs,
+                                                             guint attr_type);
+
 void                gp11_attributes_add_boolean             (GP11Attributes *attrs,
                                                              guint attr_type,
                                                              gboolean value);
@@ -157,11 +173,11 @@ gboolean            gp11_attributes_find_string             (GP11Attributes *att
 
 gboolean            gp11_attributes_find_date               (GP11Attributes *attrs,
                                                              guint attr_type,
-                                                             GDate **value);            
+                                                             GDate *value);
 
 gulong              gp11_attributes_count                   (GP11Attributes *attrs);
 
-void                gp11_attributes_ref                     (GP11Attributes *attrs);
+GP11Attributes*     gp11_attributes_ref                     (GP11Attributes *attrs);
 
 void                gp11_attributes_unref                   (GP11Attributes *attrs);
 
@@ -215,6 +231,7 @@ struct _GP11ModuleClass {
 GType               gp11_module_get_type                    (void) G_GNUC_CONST;
 
 GP11Module*         gp11_module_initialize                  (const gchar *path, 
+                                                             gpointer reserved,
                                                              GError **err);
 
 GP11ModuleInfo*     gp11_module_get_info                    (GP11Module *module);
@@ -310,6 +327,8 @@ struct _GP11SlotClass {
 
 GType               gp11_slot_get_type                      (void) G_GNUC_CONST;
 
+CK_SLOT_ID          gp11_slot_get_handle                    (GP11Slot *slot);
+
 gboolean            gp11_slot_get_reuse_sessions            (GP11Slot *slot);
 
 void                gp11_slot_set_reuse_sessions            (GP11Slot *slot, 
@@ -412,6 +431,8 @@ GType               gp11_session_get_type                   (void) G_GNUC_CONST;
 
 GP11Session*        gp11_session_from_handle                (GP11Slot *slot, CK_SESSION_HANDLE handle); 
 
+CK_SESSION_HANDLE   gp11_session_get_handle                 (GP11Session *session);
+
 GP11SessionInfo*    gp11_session_get_info                   (GP11Session *session);
 
 #if UNIMPLEMENTED
@@ -1157,6 +1178,11 @@ GType               gp11_object_get_type                    (void) G_GNUC_CONST;
 GP11Object*         gp11_object_from_handle                 (GP11Session *session, 
                                                              CK_OBJECT_HANDLE handle);
 
+GList*              gp11_objects_from_handle_array          (GP11Session *session,
+                                                             const GP11Attribute *attr);
+
+CK_OBJECT_HANDLE    gp11_object_get_handle                  (GP11Object *object);
+
 #ifdef UNIMPLEMENTED
 
 GP11Object*         gp11_object_copy                        (GP11Object *object,
@@ -1238,13 +1264,13 @@ GP11Attributes*     gp11_object_get                         (GP11Object *object,
                                                              ...);
 
 GP11Attributes*     gp11_object_get_full                    (GP11Object *object,
-                                                             guint *attr_types,
+                                                             const guint *attr_types,
                                                              gsize n_attr_types,
                                                              GCancellable *cancellable,
                                                              GError **err);
 
 void                gp11_object_get_async                   (GP11Object *object,
-                                                             guint *attr_types,
+                                                             const guint *attr_types,
                                                              gsize n_attr_types,
                                                              GCancellable *cancellable,
                                                              GAsyncReadyCallback callback,
